from .config import read_config_file
from .duckbot import DuckBot
import asyncio

async def main():
    config = read_config_file("config.yaml")
    bot_username = config["bot"]["username"]
    bot_password = config["bot"]["password"]
    device_id = config["device"]["id"]
    device_name = config["device"]["name"]
    
    duckbot = await DuckBot.create(config)

    await duckbot.run(bot_username, bot_password, device_id, device_name)

asyncio.run(main())



import asyncio
import yaml
from nio import AsyncClient, MatrixRoom, RoomMessageText, InviteEvent, Event

DEVICE_ID = "UAXKBVAPUI"

def read_config_file(config_filename):
    with open(config_filename, "r") as config_file:
        return yaml.load(config_file, yaml.FullLoader)

config = read_config_file("config.yaml")
client = AsyncClient(config["server_url"])
client.user_id = config["bot_username"]
client.access_token = config["access_token"]
client.device_id = config["old_device_id"]
client.room_id = config["room_id"]

async def message_callback(room: MatrixRoom, event: Event) -> None:
    print(event)

async def invite_callback(room: MatrixRoom, event: InviteEvent):
    await client.join(room.room_id)

async def main() -> None:
    client.add_event_callback(message_callback, Event)
    client.add_event_callback(invite_callback, InviteEvent)

    t = await client.room_send(client.room_id, "m.room.message", {"msgtype": "m.text", "body": "hejsan"})
    r = await client.encrypt(client.room_id, "m.room.message", {"msgtype": "m.text", "body": "hejsan (encrypted)"})

    await client.sync_forever(timeout=30000)

    await client.close()

asyncio.get_event_loop().run_until_complete(main())


from uuid import uuid4
import aiohttp
import json
from enum import Enum
from .matrix_client_pg_storage import MatrixClientPgStorage

class RoomStatus(Enum):
        JOINED = 1
        INVITED = 2
        KNOCKED = 3
        LEFT = 4

class MatrixClient:
    auto_join = True
    
    @classmethod
    async def create(cls, server_url, storage):
        http_client_session = aiohttp.ClientSession()
        
        return cls(server_url, http_client_session, storage)
    
    def __init__(self, server_url, http_client_session, storage: MatrixClientPgStorage,):
        self.server_url = server_url
        self.http_client = http_client_session
        self.storage = storage
        self.callbacks = {}

    def add_callback(self, event_type, callback):
        if event_type not in self.callbacks:
            self.callbacks[event_type] = []

        self.callbacks[event_type].append(callback)
    
    async def login(self, username, password, device_id, device_name):
        self.username = f"@{username}:ducksquad.io"
        
        login_request = {
            "type": "m.login.password",
            "identifier": {
                "type": "m.id.user",
                "user": username
            },
            "device_id": device_id,
            "initial_device_display_name": device_name,
            "password": password
        }
        async with self.http_client.post(f"{self.server_url}/_matrix/client/v3/login", json=login_request) as response:
            content = json.loads(await response.content.read())

            self.access_token = content["access_token"]
    
    async def send_room_message(self, room_id, message):
        event = self.__create_message_event(room_id, message)
        txn_id = uuid4()
        event_type = event["type"]
        
        await self.__put_request(f"/_matrix/client/v3/rooms/{room_id}/send/{event_type}/{txn_id}", event)
    
    async def join_room(self, room_id):
        await self.__post_request(f"/_matrix/client/v3/rooms/{room_id}/join")

    async def sync_loop(self):
        params = {}
        
        while True:
            params["timeout"] = 20000

            if "since" in self.storage.config:
                params["since"] = self.storage.config["since"]
            
            response = await self.__get_request("/_matrix/client/v3/sync", params)
            await self.__update_since(response.json_content["next_batch"])

            if "rooms" in response.json_content:
                await self.__handle_sync_events(response.json_content["rooms"])
    
    async def __update_since(self, batch_id):
        self.storage.config["since"] = batch_id
        self.storage.store()
    
    async def __handle_sync_events(self, room_events):
        if "join" in room_events:
            # TODO: Reconsider using this method and the RoomStatus to streamline handling events
            # It might be useless if the other structures are different (as with invitations)
            await self.__handle_room_events(room_events["join"], RoomStatus.JOINED)

        if "invite" in room_events:
            await self.__handle_room_invites(room_events["invite"])
    
    async def __handle_room_events(self, room_events, room_status):
        # The following is not that pretty
        for room_id in room_events:
            room = room_events[room_id]
            
            for event in room["timeline"]["events"]:
                type = event["type"]

                if type in self.callbacks:
                    for callback in self.callbacks[type]:
                        room_info = {
                            "room_id": room_id,
                            "room_status": room_status
                        }
                        await callback(event, room_info)
    
    async def __handle_room_invites(self, invites):
        if self.auto_join:
            for room_id in invites:
                await self.join_room(room_id)

    def __create_message_event(self, room_id, message):
        return {
            "type": "m.room.message",
            "sender": self.username,
            "body": message,
            "msgtype": "m.text",
            "room_id": room_id
        }

    async def __get_request(self, path, params):
        headers = self.__get_headers()
        
        async with self.http_client.get(self.__get_url(path), headers=headers, params=params) as response:
            json_content = json.loads(await response.content.read())
            response.json_content = json_content

            return response

    async def __post_request(self, path, json_body=None):
        headers = self.__get_headers();

        async with self.http_client.post(self.__get_url(path), json=json_body, headers=headers) as response:
            json_content = json.loads(await response.content.read())
            response.json_content = json_content

            return response

    async def __put_request(self, path, json_body=None):
        headers = self.__get_headers();

        async with self.http_client.put(self.__get_url(path), json=json_body, headers=headers) as response:
            json_content = json.loads(await response.content.read())
            response.json_content = json_content

            return response

    def __get_url(self, path):
        return f"{self.server_url}{path}";

    def __get_headers(self):
        headers = {}

        if self.access_token:
            headers["Authorization"] = f"Bearer {self.access_token}"

        return headers;


import enum
import string

class CommandResponseType(enum.Enum):
    MESSAGE = 1

class CommandResponse:
    type: CommandResponseType
    status: string
    command_type: string
    error: string
    content: object


import json

class MatrixClientPgStorage:
    config = {
        "since": None
    }
    
    @classmethod
    def create(cls, pg_connection):
        instance = cls(pg_connection)

        instance.__ensure_table()
        instance.load()

        return instance
    
    def __init__(self, pg_conn):
        self.pg_conn = pg_conn

    def store(self):
        with self.pg_conn.cursor() as cursor:
            # TODO: Investigate a consistent way to insert and read jsonb with psycopg2
            # rather than using string in one place
            cursor.execute(
                '''
                    UPDATE matrix_client
                    SET json = %s
                    WHERE id='config'
                ''',
                (json.dumps(self.config),))
            self.pg_conn.commit()
    
    def load(self):
        with self.pg_conn.cursor() as cursor:
            cursor.execute("SELECT * FROM matrix_client WHERE id='config'")
            existing_config = cursor.fetchone()

            if existing_config:
                self.config = existing_config[1]

    def __ensure_table(self):
        with self.pg_conn.cursor() as cursor:
            cursor.execute('''
                    SELECT * FROM information_schema.tables
                    WHERE table_name='matrix_client'
                    LIMIT 1;
                ''')
            self.pg_conn.commit()
            matched_table = cursor.fetchone()
            
            if not matched_table:
                cursor.execute('''
                    CREATE TABLE matrix_client
                    (
                        id CHAR(64) PRIMARY KEY NOT NULL,
                        json JSONB NOT NULL
                    );

                    INSERT INTO matrix_client VALUES('config', '{}');
                ''')
                self.pg_conn.commit()


from .translation_module import TranslationModule

class CommandProcessor:
    @classmethod
    async def create(cls, config):
        translator = await TranslationModule.create(config)

        return cls(translator)
    
    def __init__(self, translator):
        self.translator = translator
    
    async def parse_command(self, message):
        if not message.startswith("!"):
            return
        
        message_parts = message.split()

        match message_parts[0]:
            case "!tl":
                return await self.translator.translate(message_parts)



import psycopg2
from .matrix_client import MatrixClient
from .matrix_client_pg_storage import MatrixClientPgStorage
from .command_processor import CommandProcessor
from .command_response import CommandResponseType

class DuckBot:
    @classmethod
    async def create(cls, config):
        db_host = config["storage"]["host"]
        db_name = config["storage"]["db_name"]
        db_username = config["storage"]["username"]
        db_password = config["storage"]["password"]
        server_url = config["server"]["url"]
        
        pg_connection = psycopg2.connect(host=db_host, database=db_name, user=db_username, password=db_password)
        matrix_client_storage = MatrixClientPgStorage.create(pg_connection)
        command_processor = await CommandProcessor.create(config)
        matrix_client = await MatrixClient.create(server_url, matrix_client_storage)

        return cls(matrix_client, command_processor)

    def __init__(self, matrix_client, command_processor):
        self.matrix_client = matrix_client
        self.command_processor = command_processor

        matrix_client.add_callback("m.room.message", self.text_callback)
    
    async def run(self, username, password, device_id, device_name):
        print(f"Logging in {username}...")
        await self.matrix_client.login(username, password, device_id, device_name)
        print("Logged in")
        print("Running sync loop")
        await self.matrix_client.sync_loop()
    
    async def text_callback(self, event, room_info):
        message = event["content"]["body"]

        cmd_result = await self.command_processor.parse_command(message)

        if not cmd_result:
            return

        await self.process_command_response(cmd_result, room_info)
    
    async def process_command_response(self, cmd_result, room_info):
        match cmd_result.type:
            case CommandResponseType.MESSAGE:
                await self.matrix_client.send_room_message(room_info["room_id"], cmd_result.content)



import deepl
from .command_response import CommandResponse, CommandResponseType

class TranslationModule:
    @classmethod
    async def create(cls, config):
        auth_key = config["translator"]["auth_key"]
        deepl_translator = deepl.Translator(auth_key)

        return cls(deepl_translator)
    
    def __init__(self, deepl_translator):
        self.deepl_translator = deepl_translator
    
    async def translate(self, message_parts):
        command = message_parts[0]

        if command != "!tl":
            print(f"ERROR: Translator module received unknown command ({command})")
        
        source_lang = self.__handle_shorthands(message_parts[1])
        target_lang = self.__handle_shorthands(message_parts[2])
        text = " ".join(message_parts[3:])

        if source_lang == "." or source_lang == "?":
            source_lang = None

        try:
            translation = self.deepl_translator.translate_text(
                text,
                source_lang=source_lang,
                target_lang=target_lang)
            
            return self.create_response(
                translation.text,
                source_lang or translation.detected_source_lang,
                target_lang)
        except deepl.DeepLException as ex:
            return self.create_error_response(ex.args)
        except ValueError as ex:
            return self.create_error_response(ex.args)
        except TypeError as ex:
            return self.create_error_response(ex.args)

    def __handle_shorthands(self, language_tag):
        match language_tag:
            case "en":
                return "en-us"
            case _:
                return language_tag

    def create_response(self, text, source_lang, target_lang):
        response = CommandResponse

        response.type = CommandResponseType.MESSAGE
        response.status = "success"
        response.command_type = "translation"
        response.content = f"Translation [{source_lang.lower()}->{target_lang.lower()}]: {text}"

        return response;
    
    def create_error_response(self, error):
        response = CommandResponse
        
        response.type = CommandResponseType.MESSAGE
        response.status = "error"
        response.command_type = "translation"
        response.content = f"Translation failed: {error}"

        return response
    